@@ -2,10 +2,12 @@ from .client import Client
 from .client import make_test_client
 from .plugin import fixture
 from .plugin import make_client_server
+from .plugin import pytest_runtest_makereport
+
 from .plugin import ClientServer
 from .plugin import ClientServerConfig
 
-__version__ = "0.0.6"
+__version__ = "0.1.0"
 
 __all__ = [
     "Client",
@@ -14,4 +16,5 @@ __all__ = [
     "fixture",
     "make_client_server",
     "make_test_client",
+    "pytest_runtest_makereport",
 ]
